//==================================================
//--------- Programmed by Alexey Grebennikov--------
//-------- alexey.grebennikov@bakerhughes.com-------
//==================================================
//
module sopc (
  clk_i, sin_i, btn_i,
  sout_o, led_o, ddr_adr_o, data_o
  );
  input clk_i;                    // Input clock
  input sin_i;                    // Serial input
  input [3:0] btn_i;              // Input buttons
  output sout_o;                  // Serial output
  output [3:0] led_o;             // Drivers to the LEDs		   
  
  reg [3:0] led_o;
  // Internal registers and wires
  reg rd_r;
  reg wr_r;
  reg cs_r;
				
  reg [2:0] adr_r;
	   
  reg [31:0] counter;
  wire rst_low = ~btn_i[0];
 
  wire rxrdy_w;				
  wire txrdy_w;
  wire [7:0] datao_w;
 
	   
  wire  irq_w;
  reg [7:0] datai_r;
				   
  
  // Instantiate submodules
uart_top uart_sopc (  
     .clk_i            (clk_i), 
     .rst_i            (rst_low),
     .sin_i            (sin_i), 
     .ctsn_i           (),  
     .sout_o           (sout_o), 
     .rtsn_o           (),
     .data_i           (datai_r), 
     .rd_i             (rd_r), 
     .wr_i             (wr_r),
     .adr_i            (adr_r), 
     .cs_i             (cs_r),  
     .data_o           (datao_w), 
     .irq_o            (irq_w), 
     .txrdy_o          (txrdy_w), 
     .rxrdy_o          (rxrdy_w)
  );

  always @ (posedge clk_i)
    begin
    if (!rst_low)
        begin
          rd_r <= 1'b0;
          wr_r <= 1'b0;
          cs_r <= 1'b0;
          adr_r <= 3'b000;
 
          counter <= 32'h0; 
        end
      else if (btn_i[1] || btn_i[2])
        begin
 
          counter <= counter + 1'b1;
          if (rxrdy_w)
            begin
              if (counter === 32'h2)
                begin
                  adr_r <= 3'b000;
                  cs_r <= 1'b1;
                  rd_r <= 1'b1;
   
                end
		   
	 
              else
                begin
                  rd_r <= 1'b0;
                  cs_r <= 1'b0;
                end
            end
        end
      else if (btn_i[3])
        begin
          counter <= counter + 1'b1;
          if (counter === 32'h5)
            begin
              wr_r <= 1'b1;
              cs_r <= 1'b1;
              datai_r <= 8'h32;
            end
          else if (counter === 32'h6)
            datai_r <= 8'h33;
          else if (counter === 32'h7)
            datai_r <= 8'h34;
          else if (counter == 32'h8)
            datai_r <= 8'h35;
 
          else if ((counter > 32'h8) && (counter < 32'h20))
            datai_r <= 8'h37;
          else if (counter === 32'h20)
            datai_r <= 8'hA;
          else if (counter === 32'h21)
            datai_r <= 8'hD;
          else
            begin
              wr_r <= 1'b0;
              cs_r <= 1'b0;
              adr_r <= 3'b000;
            end
        end
      else
        begin 
          counter <= 32'h0;	   
          rd_r <=  1'b0;
          cs_r <= 1'b0;
          adr_r <= 3'b000;	 
          wr_r <= 1'b0;
        end
    end
    
    always @ (negedge clk_i)
      begin
        if (!rst_low)
          begin
            led_o <= 4'h0;
          end
        else if (counter === 32'h4)
 
          begin
            if (btn_i[1])
              led_o[3:0] <= datao_w[3:0];
            else if (btn_i[2])
              led_o[3:0] <= datao_w[7:4];
          end
        else
          begin
            if (~btn_i[1] && ~btn_i[2])
            led_o[0] <= rxrdy_w;
          end
      end
  endmodule
  